/* $Header: d:/cvsroot/tads/html/win32/tadsimg.h,v 1.2 1999/05/17 02:52:25 MJRoberts Exp $ */

/* Copyright (c) 1997 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  tadsimg.h - TADS image base class
Function
  This class serves as a base class for images stored as bitmaps.
  Subclasses are responsible for loading a specific type of image
  into our internal bitmap.
Notes
  
Modified
  11/08/97 MJRoberts  - Creation
*/

#ifndef TADSIMG_H
#define TADSIMG_H

#include <windows.h>

#ifndef HTML_OS_H
#include "html_os.h"
#endif
#ifndef TADSHTML_H
#include "tadshtml.h"
#endif

/* 
 *   drawing modes - these specify how to draw an image into a rectangle
 *   whose size varies from the size of the image 
 */
enum tadsimg_draw_mode_t
{
    /* 
     *   Draw the image at its exact size, aligning at the upper left
     *   corner of the bounding area; clip the image to fit the drawing
     *   area if the image is too big, and leave any excess area
     *   unaffected if the image is too small 
     */
    TADSIMG_DRAW_CLIP,

    /*
     *   stretch the image to fit the bounding area 
     */
    TADSIMG_DRAW_STRETCH,

    /*
     *   tile the image to fit the bounding area, starting at the upper
     *   left corner; clip the image to fit the area if it's too big 
     */
    TADSIMG_DRAW_TILE
};

/*
 *   base image class 
 */
class CTadsImage
{
public:
    CTadsImage();
    virtual ~CTadsImage();

    /* draw the image into a window */
    void draw(class CTadsWin *win, class CHtmlRect *pos,
              tadsimg_draw_mode_t mode);

    /* 
     *   Map the image's color palette into the system palette.  Returns
     *   zero if there was no change, non-zero if any colors changed.  
     */
    int map_palette(class CTadsWin *win, int foreground);

protected:
    /* AND the mask onto the screen */
    void draw_mask(HDC hdc, class CTadsWin *win, class CHtmlRect *pos,
                   tadsimg_draw_mode_t mode);
    
    /* delete any existing image */
    void delete_image();

    /* create a palette handle given an array of HTML_color_t's */
    void create_palhdl(HTML_color_t *pal, size_t palcnt);

    /* pixel buffer */
    OS_HUGEPTR(unsigned char) pix_;

    /* pixel buffer for the transparency mask, if present */
    OS_HUGEPTR(unsigned char) mask_;

    /* palette handle */
    HPALETTE palhdl_;

    /* color list making up the palette */
    HTML_color_t *colors_;
    size_t color_cnt_;

    /* size of the image */
    unsigned long width_, height_;

    /* 
     *   width of each line of the image in bytes - since Windows bitmaps
     *   must be DWORD-aligned, this may differ from the pixel width 
     */
    unsigned long width_bytes_;

    /* 
     *   bits per pixel - if this is zero, we assume that the stored image
     *   has been converted to the same bit depth as the active display 
     */
    int bpp_;
};

#endif /* TADSIMG_H */

