/* $Header$ */

/* Copyright (c) 2000 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  w32prj.h - TADS 3 debugger for 32-bit Windows - project window
Function
  
Notes
  
Modified
  01/14/00 MJRoberts  - Creation
*/

#ifndef W32PRJ_H
#define W32PRJ_H

#include <Windows.h>

#ifndef TADSHTML_H
#include "tadshtml.h"
#endif
#ifndef TADSWIN_H
#include "tadswin.h"
#endif
#ifndef W32TDB_H
#include "w32tdb.h"
#endif


/* ------------------------------------------------------------------------ */
/*
 *   Project window 
 */
class CHtmlDbgProjWin: public CTadsWin, public CHtmlDbgSubWin
{
public:
    CHtmlDbgProjWin(class CHtmlSys_dbgmain *dbgmain);
    ~CHtmlDbgProjWin();

    /* process system window creation */
    void do_create();

    /* process window destruction */
    void do_destroy();

    /* we're a child window */
    DWORD get_winstyle()
    {
        return (WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
    }
    DWORD get_winstyle_ex() { return 0; }

    /* save my window configuration */
    void save_win_config(class CHtmlDebugConfig *config,
                         const textchar_t *varname);

    /* load my window configuration */
    void load_win_config(class CHtmlDebugConfig *config,
                         const textchar_t *varname,
                         RECT *pos, int *vis);

    /* resize the window */
    void do_resize(int mode, int x, int y);

    /* activate/disactivate */
    int do_ncactivate(int flag);

    /* parent activate/disactivate */
    void do_parent_activate(int flag);

    /* gain focus */
    void do_setfocus(HWND prev_win);

    /* process a notification message */
    int do_notify(int control_id, int notify_code, LPNMHDR nmhdr);

    /* process a command */
    int do_command(int notify_code, int command_id, HWND ctl);

    /* check the status of a command */
    TadsCmdStat_t check_command(int command_id);

    /* save the configuration */
    void save_config(class CHtmlDebugConfig *config);

    /* -------------------------------------------------------------------- */
    /*
     *   CHtmlDbgSubWin implementation 
     */
    
    /* check the status of a command routed from the main window */
    virtual TadsCmdStat_t active_check_command(int command_id);

    /* execute a command routed from the main window */
    virtual int active_do_command(int notify_code, int command_id,
                                  HWND ctl);

    /* get my window handle */
    virtual HWND active_get_handle() { return handle_; }

    /* get the selection range - we don't have one */
    virtual int get_sel_range(class CHtmlFormatter **formatter,
                              unsigned long *sel_start,
                              unsigned long *sel_end)
    {
        return FALSE;
    }

    /* 
     *   generate the display and configuration group names for an
     *   external resource file 
     */
    static void gen_extres_names(char *display_name,
                                 char *config_group_name, int fileno);

    /* clear project information from a configuration */
    static void clear_proj_config(class CHtmlDebugConfig *config);

    /* reset the project window - clear all sources, includes, etc */
    void reset_proj_win();

protected:
    /* create the tree control */
    void create_tree();

    /* hook procedure for subclassed tree control */
    static LRESULT CALLBACK tree_proc(HWND hwnd, UINT msg, WPARAM wpar,
                                      LPARAM lpar);

    /* method version of subclassed tree hook */
    LRESULT do_tree_proc(HWND hwnd, UINT msg, WPARAM wpar, LPARAM lpar);

    /* process a drop-files event in the tree control */
    void do_tree_dropfiles(HDROP hdrop);

    /* insert an item into our tree view control after the given item */
    HTREEITEM insert_tree(HTREEITEM parent, HTREEITEM ins_after,
                          const textchar_t *txt, DWORD state,
                          int icon_index, struct proj_item_info *info);

    /* insert an item in our tree view control at the end of a list */
    HTREEITEM append_tree(HTREEITEM parent, const textchar_t *txt,
                          DWORD state, int icon_index,
                          struct proj_item_info *info)
    {
        /* simply insert the item after the last child of the parent */
        return insert_tree(parent, TVI_LAST, txt, state, icon_index, info);
    }

    /* prompt the user for a file, and add it to the project */
    void add_project_file();

    /* prompt the user for a folder, and add it to the project */
    void add_project_dir();

    /* add an external resource file */
    void add_ext_resfile();

    /* delete the current selection in the tree */
    void delete_tree_selection();

    /* add a file to the project */
    HTREEITEM add_file_to_project(HTREEITEM parent, HTREEITEM after,
                                  const char *fname);

    /* open an item */
    void open_item(HTREEITEM item);

    /* open an item in the text editor */
    void open_item_in_text_editor(HTREEITEM item);

    /* get the path to an item */
    void get_item_path(char *buf, size_t buflen, HTREEITEM item);

    /* get the directory that will contain children of an item */
    void get_item_child_dir(char *buf, size_t buflen, HTREEITEM item);

    /* get the proj_item_info for a tree item */
    struct proj_item_info *get_item_info(HTREEITEM hitem)
    {
        TV_ITEM item;

        /* if the item is null, there's no info */
        if (hitem == 0)
            return 0;
        
        /* ask the tree to fill in the TV_ITEM structure for us */
        item.hItem = hitem;
        item.mask = TVIF_PARAM;
        TreeView_GetItem(tree_, &item);

        /* return the lParam from the TV_ITEM - this is our info pointer */
        return (proj_item_info *)item.lParam;
    }

    /* 
     *   fix up the external resource file numbers - this should be called
     *   whenever we insert or delete an external resource file 
     */
    void fix_up_extres_numbers();

    /* set an external resource item's in-install icon */
    void set_extres_in_install(HTREEITEM item, int in_install);

    /* load files from the configuration into a given container */
    void load_config_files(class CHtmlDebugConfig *config, HTREEITEM parent);

    /* save files from the given section into the configuration */
    void save_config_files(class CHtmlDebugConfig *config, HTREEITEM parent);

    /* debugger main window object */
    class CHtmlSys_dbgmain *dbgmain_;

    /* 
     *   flag: a project is open in the debugger (if not, we offer only
     *   minimal user interface, since if there's no project open there's
     *   really nothing the project window is good for) 
     */
    unsigned int proj_open_ : 1;

    /* my tree view control */
    HWND tree_;

    /* tree item for the 'include files' group */
    HTREEITEM inc_item_;

    /* context menu for project window */
    HMENU ctx_menu_;

    /* context menu container */
    HMENU ctx_menu_container_;

    /* original window procedure for the tree control */
    WNDPROC tree_defproc_;

    /* image list */
    HIMAGELIST img_list_;

    /* state image list */
    HIMAGELIST state_img_list_;

    /* indices of our images in the image list */
    int img_idx_root_;
    int img_idx_folder_;
    int img_idx_source_;
    int img_idx_resource_;
    int img_idx_extres_;
    int img_idx_res_folder_;
    int img_idx_in_install_;
    int img_idx_not_in_install_;
};

#endif /* W32PRJ_H */

